/*******************************************************************
 * FILE: xutils.c - utility functions for Motif GUI     	   *
 *								   *
 * Author: Peter Rajcani, except as noted			   *
 *******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "xm.h"
#include "resource.h"

#define MAXARGS     	20
#define NBUTTONS    	10
#define MAXFIELDS	10
#define HORIZ		1
#define VERT		2


extern Widget	top_level, board;
extern int	resource_file, image_flag, xv_disp_flag;
Widget		editor[MAXFIELDS];

/************************************************************
 * Menu package - code taken from "X-windows programming, 
 * OSF/Motif edition" by D.Young.
 ************************************************************/

void xs_create_menu_buttons (title, menu, menulist, nitems)
     char	*title;
     Widget	menu;
     xs_menu_struct  *menulist;
     int	nitems;
{
  Arg	      wargs[1];
  int	      i;
  WidgetList  buttons;
  int	      separators = 0;

  buttons  = (WidgetList) XtMalloc (nitems * sizeof (Widget));

  if (title) {
     XtCreateManagedWidget (title, xmLabelWidgetClass, menu, NULL, 0);
     XtCreateManagedWidget ("separator", xmSeparatorWidgetClass, 
                            menu, NULL, 0);
  }

  for (i=0; i<nitems; i++) {
      if (menulist[i].name == NULL) {
	 XtCreateManagedWidget ("separator",
				xmSeparatorWidgetClass,
				menu, NULL, 0);
	 separators++;
       }
       
       else if (menulist[i].func) {
	       buttons[i-separators] = XtCreateWidget (menulist[i].name,						       xmPushButtonWidgetClass,
					       menu, NULL, 0);
	       XtAddCallback(buttons[i-separators], XmNactivateCallback,
			     menulist[i].func, menulist[i].data);
       }

       else if (!menulist[i].sub_menu)
	       buttons[i-separators] = XtCreateWidget (menulist[i].name,
						     xmLabelWidgetClass,
 						     menu, NULL, 0);

	else {
	     Widget sub_menu;
	     sub_menu = XmCreatePulldownMenu (menu,
		        ((menulist[i].sub_menu_title == NULL)? "submenu":
		         menulist[i].sub_menu_title), NULL, 0);
	     XtSetArg (wargs[0], XmNsubMenuId, sub_menu);
	     buttons[i-separators] =
		XtCreateWidget (menulist[i].name,
				xmCascadeButtonWidgetClass,
				menu, wargs, 1);
	     xs_create_menu_buttons (menulist[i].sub_menu_title,
				     sub_menu, menulist[i].sub_menu,
				     menulist[i].n_sub_items);
	     }
    } 
    XtManageChildren (buttons, nitems - separators);
}



void ok_callback (w, client_data, call_data)
	Widget		    w;
	caddr_t 	    client_data;
	XmAnyCallbackStruct *call_data;

{
     XtUnmanageChild(w);
}


void error (errmessage)
   char     *errmessage;
{
   Widget   message_box;
   Arg	    wargs[2];
   XmString xmstr;
   void	    ok_callback ();

    xmstr = XmStringCreate (errmessage, XmSTRING_DEFAULT_CHARSET);
    XtSetArg (wargs[0], XmNmessageString, xmstr);
    /*message_box = XmCreateMessageDialog (board, "Error message", wargs, 1);*/
    message_box = XmCreateWarningDialog (board, "Error message", wargs, 1);
    XtAddCallback (message_box, XmNokCallback, ok_callback, NULL);
    XtManageChild(message_box);
    free (xmstr);
}
  

/* fs_cancel: user has selected cancel button in the file selection box */

void fs_cancel (w, dialog_box, call_data)
       Widget 	w;
       Widget	dialog_box;
       XmSelectionBoxCallbackStruct *call_data;
{
      XtUnmanageChild (dialog_box);
}


/* select_file : Create a file selection box widget */

Widget select_file (parent, fs_ok, args, n)
       Widget    parent;
       void 	 (*fs_ok) ();
       Arg	 *args;
       int	 n;
{
    Widget	fs;
    void	fs_cancel ();

    fs = XmCreateFileSelectionDialog (parent, "select file",args, n);
    XtAddCallback (fs, XmNokCallback, fs_ok, fs);
    XtAddCallback (fs, XmNcancelCallback, fs_cancel, fs);
    return fs;
}


/* create_dialog: create a simple popup dialog widget
 * with one editing field.
 */

Widget create_dialog (parent, label)
         Widget   parent;
	 char	  *label;
{
   Widget       dialog;
   Arg   	wargs[MAXARGS];
   int   	nargs;
   XmString  	xmstr;	
   
   dialog = XmCreatePromptDialog (parent, "dialog box",
				  NULL, 0);

   xmstr = XmStringCreate (label, XmSTRING_DEFAULT_CHARSET);
   nargs = 0;
   XtSetArg (wargs[nargs], XmNselectionLabelString, xmstr); nargs++;
   XtSetValues (dialog, wargs, nargs);
   XtAddCallback (dialog, XmNcancelCallback, fs_cancel, dialog);
   free (xmstr);
   return dialog;
}


void quit_ok (w, client_data, call_data)
	Widget		    w;
	caddr_t 	    client_data;
	XmAnyCallbackStruct *call_data;
{
    XtCloseDisplay (XtDisplay(top_level));
    if (xv_disp_flag) {
	system ("xvkill");
        system ("rm tmp");
    }
    exit(0);
}


void quit_cancel (w, client_data, call_data)
	Widget		    w;
	caddr_t 	    client_data;
	XmAnyCallbackStruct *call_data;
{
    XtUnmanageChild(w);
}


void quit (widget, client_data, call_data)
	Widget  widget;
	caddr_t client_data;
	XmAnyCallbackStruct *call_data;
{
   Widget   message_box;
   Arg	    wargs[10];
   int	    nargs;
   XmString xmstr, yes_str, no_str;
   void	    quit_ok (), quit_cancel ();

    xmstr = XmStringCreate ("Do you really want to quit?", 
                             XmSTRING_DEFAULT_CHARSET);
    yes_str = XmStringCreate ("Yes",
                             XmSTRING_DEFAULT_CHARSET);
    no_str = XmStringCreate ("No",
                             XmSTRING_DEFAULT_CHARSET);
    nargs = 0;
    XtSetArg (wargs[nargs], XmNmessageString, xmstr); nargs++;
    XtSetArg (wargs[nargs], XmNokLabelString, yes_str); nargs++;
    XtSetArg (wargs[nargs], XmNcancelLabelString, no_str); nargs++;
    message_box = XmCreateQuestionDialog (board, "Message", wargs, nargs);
    XtAddCallback (message_box, XmNokCallback, quit_ok, NULL);
    XtAddCallback (message_box, XmNcancelCallback, quit_cancel, NULL);
    XtManageChild(message_box);
    free (xmstr); free (yes_str); free (no_str);
}
 
/* display File Selection Box */

void display_file_sb (label, mask, ok_callback)
   char		*label, *mask;
   void		(*ok_callback) ();
{
   Widget 	file_select_box;
   Arg	  	wargs[2];
   int		n;
   XmString 	xmlabel, xmmask;

   n = 0;
   if (label != NULL) {
       xmlabel = XmStringLtoRCreate (label, XmSTRING_DEFAULT_CHARSET);
       XtSetArg ( wargs[n], XmNfilterLabelString, xmlabel ); n++; 
   }

   if (mask != NULL) {
       /* set the directory mask */
       xmmask = XmStringLtoRCreate (mask, XmSTRING_DEFAULT_CHARSET);
       XtSetArg ( wargs[n], XmNdirMask, xmmask ); n++;
   }

   file_select_box  = select_file (board, ok_callback, wargs, n);
   XtManageChild (file_select_box);
   if (mask != NULL) free (xmmask);
   if (label != NULL) free (xmlabel);
}


void display_dialog (label, ok_callback)
   char		*label;
   void		(*ok_callback) ();
{
  Widget      prompt_box;

  prompt_box = create_dialog (board, label);
  XtAddCallback (prompt_box, XmNokCallback, ok_callback, prompt_box);
  XtManageChild (prompt_box);
}


/* Create a dialog form with buttons arranged vertically */

Widget create_button_dialog (title, label, buttons, nbuttons, name)
  char	    *title, *label;
  button    *buttons;
  int	    nbuttons;
  char 	    *name;
{
  int  	    i, n, pos[NBUTTONS+1];
  Widget    labelw, buttonw[NBUTTONS], dialog;  
  Arg	    wargs[10];
  XmString  xmstr;

  
  nbuttons = (nbuttons <= NBUTTONS) ? nbuttons : NBUTTONS;

  dialog = XmCreateFormDialog (board, title, NULL, 0);
  if (label != NULL) {
        xmstr = XmStringLtoRCreate (label,
                                     XmSTRING_DEFAULT_CHARSET);
	n = 0;
	XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
        labelw = XtCreateWidget ("label", xmLabelWidgetClass, dialog,
				wargs, n);
  } else 
        labelw = XtCreateWidget ("label", xmLabelWidgetClass, dialog,
				NULL, 0);

	for (i=0; i<nbuttons; i++) {
	    buttonw[i] = XtCreateWidget (buttons[i].label,
					xmPushButtonWidgetClass,
					dialog, NULL, 0);
	    XtAddCallback (buttonw[i], XmNactivateCallback, 
                           buttons[i].func, name);
        }
	 
	/*
         * Set the resources to make dialog look like this
         *   	label ("Please .... ")
         *	button one  
	 *	button two 
         *        .
         *        .
         *        .
	 */

	pos[0] = 0;
	for (i=0; i<= nbuttons; i++) {
	    pos[i+1] = 99/(nbuttons+1) * (i+1);
	    n=0;
	    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_POSITION); n++;
	    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_POSITION); n++;
	    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
            XtSetArg (wargs[n], XmNleftPosition, 1); n++;
            XtSetArg (wargs[n], XmNrightPosition, 99); n++;
	    XtSetArg (wargs[n], XmNtopPosition, pos[i]+1); n++;
	    XtSetArg (wargs[n], XmNbottomPosition, pos[i+1]); n++;

	    if (i == 0)
                XtSetValues (labelw, wargs, n);
	    else
	        XtSetValues (buttonw[i-1], wargs, n);
        }

	XtManageChild (labelw);
	XtManageChildren (buttonw, nbuttons);
	if (label != NULL) free (xmstr);
	return dialog;
}

void set_label (label, str)
Widget	label;
char	*str;
{
  Arg	wargs[2];
  XmString  xmstr;

  xmstr = XmStringCreate (str, XmSTRING_DEFAULT_CHARSET);
  XtSetArg (wargs[0], XmNlabelString, xmstr); 
  XtSetValues (label, wargs, 1);
  free (xmstr);
}

/***************************************************************
 * Function: create_dialog_box
 *
 * Creates a multi-purpose dialog box with edit fields, radio buttons
 * and toggle buttons.
 ***************************************************************/

void create_dialog_box (title, edit_labels, text, nfields, 
			  radio_label, radio_buttons, nradio, 
			  toggle_label, toggle_buttons, ntoggle,  
			  ok_callback)

char	*title;
char	*edit_labels[], *text[]; 
int	nfields;
char	*radio_label;
radio_button  *radio_buttons;
int	nradio;
char	*toggle_label;
toggle_button  *toggle_buttons;
int	ntoggle;
void    (*ok_callback) ();
{
  Widget   bb, edit_label[MAXFIELDS] , toggle_buttonw[NBUTTONS], 
	   radio_buttonw[NBUTTONS], ok_button, cancel_button, 
           separator, box_label, toggle_labelw, radio_box;
  Arg	   wargs[10];
  int	   i, n, x, y, maxw, height,  stepx, stepy = 10;
  int	   button_start, button_last, button_top; 
  int	   xpos, ypos;
  char     buttonName[20], fieldStr[10], labelStr[20];
  XmString xmstr;
  void	   cancel ();
  Dimension width, bwidth;


     nfields = ( nfields < MAXFIELDS ) ? nfields : MAXFIELDS;
     ntoggle = ( ntoggle < NBUTTONS ) ? ntoggle : NBUTTONS;
     nradio = ( nradio < NBUTTONS ) ? nradio : NBUTTONS;


/*
     if (arrange == VERT) {
	width = (labelw > EDIT_WIDTH ) ? labelw  + 20: EDIT_WIDTH + 20;
        ewidth = labelw;
     } else {
	width = labelw + EDIT_WIDTH + 20;
	ewidth = EDIT_WIDTH;
     }
*/

     /* determine the size of the container widget */
     x = 1; y = 10;
/*
     if (ntoggle > 0 && nradio > 0)
	    width = (width > DIALOG_WIDTH) ? width : DIALOG_WIDTH;
*/

     n = (nradio > ntoggle) ? nradio : ntoggle;
     if (radio_label != NULL || toggle_label != NULL) {
         height = y + LABEL_HEIGHT + 1;
     } else {
         height = y;
     }

/* 
     if (arrange == VERT)
         height += (n + 1)*(BUTTON_HEIGHT + 1)+ (EDIT_HEIGHT + LABEL_HEIGHT + 1 + stepy) * nfields;

     else
         height += (n + 1)*(BUTTON_HEIGHT + 1)+ (EDIT_HEIGHT + stepy) * nfields;
*/

     height += (n + 1)*(BUTTON_HEIGHT + 1)+ (EDIT_HEIGHT + LABEL_HEIGHT + 1 + stepy) * nfields;
     if (n == 0) height += 3*stepy;
     else height += 4*stepy;

     /*printf ("n = %d, width = %d, height = %d\n", n, width, height);*/

     /* create a popup shell to hold the dialog box */ 

         bb = XmCreateBulletinBoardDialog (board, "wftdialog", NULL, 0);

     if (resource_file) {
         XtSetArg (wargs[0], XmNwidth, &width);
         XtGetValues (bb, wargs, 1);
         printf ("width = %d\n", width);
         n = 0;
     } else {
          width = DIALOG_WIDTH;
          n = 0;
          XtSetArg (wargs[0], XmNwidth, DIALOG_WIDTH); n++; 
     }

     XtSetArg (wargs[n], XmNautoUnmanage, FALSE); n++;
     XtSetArg (wargs[n], XmNheight, height); n++;
     xmstr = XmStringCreate (title, XmSTRING_DEFAULT_CHARSET);
     XtSetArg (wargs[n], XmNdialogTitle, xmstr); n++;
     XtSetValues (bb, wargs, n);

     free (xmstr);
     
     /* text editing fields */
     if (nfields > 0) {

     	for (i = 0; i < nfields; i++) {
	    sprintf (labelStr, "label%d", i+1);
	    sprintf (fieldStr, "field%d", i+1);
     	    n = 0;
     	    /*XtSetArg (wargs[n], XmNwidth, labelw); n++;
     	    XtSetArg (wargs[n], XmNheight, LABEL_HEIGHT); n++;*/
     	    XtSetArg (wargs[n], XmNx, x); n++;
     	    XtSetArg (wargs[n], XmNy, y); n++;
     	    edit_label[i] = XtCreateWidget (labelStr, 
					    xmLabelWidgetClass, bb,
			                    wargs, n);

/*
	    xpos = (arrange == VERT) ? x : x + labelw + 20;
	    ypos = (arrange == VERT) ? y + LABEL_HEIGHT : y;
*/
	    xpos = x; ypos = y + LABEL_HEIGHT;
            n = 0;
            if (!resource_file) {
     	        XtSetArg (wargs[n], XmNwidth, EDIT_WIDTH); n++;
     	        XtSetArg (wargs[n], XmNheight, EDIT_HEIGHT); n++;
	    }
     	    XtSetArg (wargs[n], XmNx, xpos); n++;
     	    XtSetArg (wargs[n], XmNy, ypos); n++;
     	    editor[i] = XtCreateWidget (fieldStr, xmTextWidgetClass, 
					bb, wargs, n);

/*
	    if (arrange == VERT)
      	       y += EDIT_HEIGHT + LABEL_HEIGHT + stepy;
	    else
      	       y += EDIT_HEIGHT + stepy;
*/
      	     y += EDIT_HEIGHT + LABEL_HEIGHT + stepy;

            if (text != NULL) {
                if (text[i] != NULL)
                    XmTextSetString (editor[i], text[i]);
            }
            set_label (edit_label[i], edit_labels[i]);
        }
     }

     button_top = y;

     /* create radio box with buttons */
     
     if (nradio > 0) {

     	/* radio box label */

	if (radio_label != NULL) {
     	    n = 0;
            /*XtSetArg (wargs[n], XmNwidth, LABEL_WIDTH); n++;*/
     	    XtSetArg (wargs[n], XmNheight, LABEL_HEIGHT); n++;
     	    XtSetArg (wargs[n], XmNx, x); n++;
            XtSetArg (wargs[n], XmNy, y); n++;
            box_label = XtCreateWidget ("box_label", xmLabelWidgetClass, bb,
			         wargs, n);
     	    y += LABEL_HEIGHT+1;

            set_label (box_label, radio_label);

         } else {
            if (toggle_label != NULL) y += LABEL_HEIGHT + 1;
         }

          button_start = y;

     	  n = 0;
     	  XtSetArg (wargs[n], XmNy, y); n++;
     	  XtSetArg (wargs[n], XmNx, x); n++;
     	  radio_box = XmCreateRadioBox (bb, "radio_box", wargs, n);
     
	  maxw = 0;
     	  for (i = 0; i < nradio; i++) {
	  
     	    n = 0;
	    sprintf (buttonName, "radio button%d", i+1);

     	    xmstr = XmStringCreate (radio_buttons[i].label, 
				  XmSTRING_DEFAULT_CHARSET);
     	    XtSetArg (wargs[n], XmNy, y+1); n++;
     	    XtSetArg (wargs[n], XmNx, x+1); n++;
	    XtSetArg (wargs[n], XmNindicatorType, XmONE_OF_MANY); n++;
     	    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
	    XtSetArg (wargs[n], XmNset, radio_buttons[i].set); n++;
     	    radio_buttonw[i] = XtCreateManagedWidget (buttonName, 
		            	     xmToggleButtonWidgetClass, 
				     radio_box, wargs, n);
            XtAddCallback (radio_buttonw[i], XmNvalueChangedCallback, 
			 	 radio_buttons[i].func, 
				 (XtPointer) radio_buttons[i].type);
     	    free (xmstr);
     	    y += BUTTON_HEIGHT+1;

	    
          }
	  button_last = y;
     }

     /* toggle buttons */

     if (ntoggle > 0) {

        y = button_top; 

        /* create label if needed */
	if (toggle_label != NULL) {
     	    n = 0;
            XtSetArg (wargs[n], XmNwidth, LABEL_WIDTH); n++;
     	    XtSetArg (wargs[n], XmNheight, LABEL_HEIGHT); n++;
     	    XtSetArg (wargs[n], XmNx, x); n++;
            XtSetArg (wargs[n], XmNy, y); n++;
            toggle_labelw = XtCreateWidget ("toggle label", 
					     xmLabelWidgetClass, bb,
					     wargs, n);
     	    y += LABEL_HEIGHT+1;

            set_label (toggle_labelw, toggle_label);
         }

         if (nradio > 0) y = ( y < button_start ) ? button_start : y;

         maxw = 0;
         for (i = 0; i < ntoggle; i++) {
	  
     	  	n = 0;
	  	sprintf (buttonName, "toggle button%d", i+1);

     	  	xmstr = XmStringCreate (toggle_buttons[i].label, 
				  	XmSTRING_DEFAULT_CHARSET);
     	  	XtSetArg (wargs[n], XmNy, y+1); n++;
     	  	XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
	      	XtSetArg (wargs[n], XmNset, toggle_buttons[i].set); n++;
     	  	toggle_buttonw[i] = XtCreateManagedWidget (buttonName, 
		            xmToggleButtonWidgetClass, bb, wargs, n);
          	XtAddCallback (toggle_buttonw[i], 
				XmNvalueChangedCallback, 
			 	toggle_buttons[i].func, bb);
     	  	free (xmstr);
     	  	y += BUTTON_HEIGHT + 1;

                /* find the width of the largest toggle button */
                if (nradio > 0) {
	            XtSetArg (wargs[0], XmNwidth, &bwidth);
	            XtGetValues (radio_buttonw[i], wargs, 1);
	            if (bwidth > maxw) maxw = bwidth;
                }
     	  }
         
           if (nradio > 0)
		x = (bwidth + 10 > width/2) ? width - (bwidth + 10) : width/2 - 20 ;

     	    XtSetArg (wargs[0], XmNx, x); 
            for (i = 0; i < ntoggle; i++)
		XtSetValues (toggle_buttonw[i], wargs, 1);

          if (nradio > 0) y = (y > button_last) ? y : button_last;
      }


     /*** separator **/

     if (ntoggle > 0 || nradio > 0) y += stepy;
     n = 0;
     XtSetArg (wargs[n], XmNy, y); n++;
     XtSetArg (wargs[n], XmNwidth, width-20); n++;
     separator = XtCreateManagedWidget ("separator", 
					xmSeparatorWidgetClass,
					bb, wargs, n);

     /* OK & Cancel buttons */
     stepx = (width - 2*BUTTON_WIDTH)/3;
     n = 0;
     if (!resource_file) {
         XtSetArg (wargs[n], XmNwidth, BUTTON_WIDTH); n++;
         XtSetArg (wargs[n], XmNheight, BUTTON_HEIGHT); n++;
     }
     xmstr = XmStringCreate ("OK", XmSTRING_DEFAULT_CHARSET);
     XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
     XtSetArg (wargs[n], XmNx, stepx); n++;
     XtSetArg (wargs[n], XmNy, y + 2*stepy); n++;

     ok_button = XtCreateManagedWidget ("ok_button",xmPushButtonWidgetClass,
				         bb, wargs, n);

     n = (resource_file) ? 0 : 2;
     xmstr = XmStringCreate ("Cancel", XmSTRING_DEFAULT_CHARSET);
     XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
     XtSetArg (wargs[n], XmNx, 2*stepx + BUTTON_WIDTH); n++;
     XtSetArg (wargs[n], XmNy, y + 2*stepy); n++;

     cancel_button = XtCreateManagedWidget ("cancel_button",
					    xmPushButtonWidgetClass,
				            bb, wargs, n);
     XtAddCallback (ok_button, XmNactivateCallback, ok_callback, bb);
     XtAddCallback (cancel_button, XmNactivateCallback, cancel, bb);

     /*printf ("actual height = %d\n", y + 3*stepy + BUTTON_HEIGHT );*/

     /* Manage the widgets */

     if (nfields > 0) {
        XtManageChildren (editor, nfields);
        XtManageChildren (edit_label, nfields);
     }

     if (nradio > 0) {
     	if (radio_label != NULL) XtManageChild (box_label);
     	XtManageChildren (radio_buttonw, nradio);
        XtManageChild (radio_box);
      }

     if (ntoggle > 0) {
     	if (toggle_label != NULL) XtManageChild (toggle_labelw);
        XtManageChildren (toggle_buttonw, ntoggle);
     }

     XtManageChild (bb);
     free (xmstr);
}

void cancel (widget, dialog, call_data)
Widget		widget;
Widget		dialog;
XmAnyCallbackStruct call_data;
{
  XtUnmanageChild (dialog);
}


void crop_name (name)
char	**name;
{
  char *ptr;
  int  length, l;

  length = strlen (*name);

  for (ptr = *name + length - 1, l = length; *ptr != '/' && l > 0; ptr--, l--);

  *name = ptr + 1;
   printf ("Filename: %s\n", *name);
}
  
void flush_Xbuffer ()
{
}
